#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"
#include "xparameters.h"

#include "ether.h"
#include "flash.h"
#include "audio.h"
#include "menu.h"


void main_init();				//function declaration
void audio_sampler_handler(void *callback);
void uart_handler(void *callback);

volatile short uart_interrupt_count=0;
char uart_character;
unsigned long uart_out;
volatile unsigned short new_data;
volatile int out_enable=0;

void FlashRecord()				//record audio to flash
{
  int i = 0;
  unsigned long sample_count = 0;
  unsigned long max_count = 128000;
  unsigned char byte;

  while (sample_count < max_count)
  {
      /* new_data is set in the interrupt handler when new data is ready. */
      if(new_data == 1) {

	  new_data = 0;

	  /* grab the data from the audio controller*/
	  uart_out = XIo_In32(XPAR_AUDIO_SAMPLER_BASEADDR);
	  /* the standard audio format is little endian, so we swap the bytes:
	  we have 4 bytes, they get reordered from 1234 to 2143. */
	  uart_out = ((uart_out >> 24 & 0xff) << 16) |
	  ((uart_out >> 16 & 0xff) << 24) |
	  (uart_out >> 8 & 0xff) |
	  ((uart_out & 0xff) << 8);


        sample_count++;
        // Take 10 sample data from Audio, and save in SRAM
          // Save lower 16 bit
          //sram_buffer[2*sample_count] = sampler_out;
          // Save upper 16 bit
          //sram_buffer[2*sample_count+1] = sampler_out>>16;

     // write the 0th 8 bit
     //byte = flash_buffer[i*4+1];
     //word = word << 8 | byte;
     byte = uart_out & 0xff;
     FlashProgram(FLASH_BASE_ADDR+(sample_count<<2)+1, byte);

     // write the 1th 8 bit
     //byte = flash_buffer[i*4];
     //word = word << 8 | byte;
     byte = uart_out>>8;
     FlashProgram(FLASH_BASE_ADDR+(sample_count<<2), byte);

     // write the 2th 8 bit
     //byte = flash_buffer[i*4+3];
     //word = word << 8 | byte;
     byte = uart_out>>16;
     FlashProgram(FLASH_BASE_ADDR+(sample_count<<2)+3, byte);


          // write the 3th 8 bit
     //byte = flash_buffer[i*4+2];
     //word = byte;
     byte = uart_out>>24;
     FlashProgram(FLASH_BASE_ADDR+(sample_count<<2)+2, byte);
     }
  }
}

void FlashOut()					//playing audio saved in flash before
{
  int i;
  unsigned long max_count = 128000;
  volatile unsigned char *flash_buffer;
  unsigned char byte;
  unsigned long word;

  flash_buffer = (volatile unsigned char *) FLASH_BASE_ADDR;

  for (i = 0; i < max_count ; )
  {
  ////////////////////////////////////////
  // 32bit: 3/2/1/0
  ////////////////////////////////////////
     // read the 3th(highest) 8 bit

     if(new_data == 1) {
     i ++;
     new_data = 0;
     byte = flash_buffer[(i<<2)+2];
     word = byte;

     // read the 2th 8 bit
     byte = flash_buffer[(i<<2)+3];
     word = word << 8 | byte;

     // read the 1th 8 bit
     byte = flash_buffer[i<<2];
     word = word << 8 | byte;

     // read the 0th 8 bit
     byte = flash_buffer[(i<<2)+1];
     word = word << 8 | byte;
     output_sample((WORD *)(&word));
     }
  }

}


int main()
{
  uart_interrupt_count = 0;
  new_data = 0;

  /////////////////////////////////////////////////////////////////////////////
  // Setup Interrupt
  /////////////////////////////////////////////////////////////////////////////
  /* Enable UART and Audio interrupts and register the ISR */
  XIntc_RegisterHandler( XPAR_INTC_BASEADDR, XPAR_INTC_MYUART_INTERRUPT_INTR,
	 		 (XInterruptHandler)uart_handler, (void *)0);
  XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR, XPAR_INTC_AUDIO_SAMPLER_INTERRUPT_INTR,
			 audio_sampler_handler, (void*)0);

  XIntc_mEnableIntr( XPAR_INTC_BASEADDR, XPAR_MYUART_INTERRUPT_MASK | XPAR_AUDIO_SAMPLER_INTERRUPT_MASK);
  //XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_AUDIO_SAMPLER_INTERRUPT_MASK);

  XIntc_mMasterEnable( XPAR_INTC_BASEADDR );
  XIntc_Out32(XPAR_INTC_BASEADDR + XIN_MER_OFFSET, XIN_INT_MASTER_ENABLE_MASK);
  microblaze_enable_interrupts();
  XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);

  main_init();	//initialization
  putMenu();	//display menu on VGA

  while (1)
  {
      if ( uart_interrupt_count  != 0) {

	 vga_menu(uart_character);		//call the vga_menu if there is input from uart
         uart_interrupt_count = 0;

      }

      /* new_data is set in the interrupt handler when new data is ready. */
      if(new_data == 1) {

	      new_data = 0;

	      /* grab the data from the audio controller*/
	      uart_out = XIo_In32(XPAR_AUDIO_SAMPLER_BASEADDR);
	      /* the standard audio format is little endian, so we swap the bytes:
	      we have 4 bytes, they get reordered from 1234 to 2143. */
	      uart_out = ((uart_out >> 24 & 0xff) << 16) |
	      ((uart_out >> 16 & 0xff) << 24) |
	      (uart_out >> 8 & 0xff) |
	      ((uart_out & 0xff) << 8);
	      if(out_enable==0) uart_out=0x00000000;
	      /* output the new sample to the ethernet card */
	      output_sample((WORD *)(&uart_out));
     }
  }

  return 0;
}
